<?php

namespace app\abutment\logic;

use app\abutment\model\SupplierUser as SupplierUserModel;
use app\abutment\model\SupplierRelationUser as SupplierRelationUserModel;
use app\abutment\model\SupplierUser;
use think\Exception;
use think\facade\Cache;
use think\facade\Config;
use think\facade\Db;
use think\helper\Str;


//供应商账号管理
class Account
{

    //登录
    public static function login(array $param = [])
    {

        $db = new SupplierUserModel();

        $res = $db->where([
            'is_del' => $db::$is_del_normal,
            'mobile' => $param['mobile']
        ])
            ->findOrEmpty()
            ->toArray();

        if (empty($res)) return json_show(1004, '该账号不存在');
        if ($res['status'] == $db::$status_disabled) return json_show(1004, '该账号已禁用');

        $password = get_encryption_password($param['password'], $res['salt']);
        if ($password['password'] != $res['password']) return json_show(1004, '密码错误');

        //更新token
        $token = Str::random(40, -1);

        $expire_int = mt_rand(3600, 7200);
        $expire_time = date('Y-m-d H:i:s', time() + $expire_int);

        $rs = $db
            ->where($db->getPk(), $res[$db->getPk()])
            ->save(['token' => $token, 'expire_time' => $expire_time]);

        if (!$rs) throw new Exception('更新账号token信息失败');

        $list = SupplierRelationUserModel::field('id,supplierNo,supplierName')
            ->where([
                'is_del' => $db::$is_del_normal,
                'uid' => $res['uid'],
            ])
            ->select()
            ->toArray();

        $info = [
            'uid' => $res['uid'],
            'nickname' => $res['nickname'],
            'mobile' => $res['mobile'],
            'email' => $res['email'],
            'token' => $token,
            'expire_time' => $expire_time,
            'supplier_list' => $list,
        ];

        //过期时间指定一个具体的日期的时候，还没到这个日期缓存就过期了，指定一个秒数，可以符合预期，邪门儿
        Cache::set(Config::get('redis_key.user_info_token') . $token, json_encode($info), $expire_int);

        return json_show(0, '登录成功', $info);


    }

    //获取供应商账号列表
    public static function getAccountList(array $param = [])
    {

        $db = new SupplierUserModel();


        $rs = $db
            ->where('is_del', $db::$is_del_normal);

        if ($param['keyword'] != '') $rs->whereLike('nickname|mobile', '%' . $param['keyword'] . '%');
        if ($param['status'] != '') $rs->where('status', $param['status']);
        if ($param['supplierNo'] != '') {
            $uids = SupplierRelationUserModel::where([
                'is_del' => $db::$is_del_normal,
                'supplierNo' => $param['supplierNo']
            ])->column('uid');

            $rs->whereIn('uid', $uids);
        }

        $count = $rs->count('uid');


        $list = $rs
            ->field('uid,nickname,mobile,email,status,addtime,creater')
            ->order('addtime', 'desc')
            ->page($param['page'], $param['size'])
            ->append(['supplier_list'])
            ->withAttr('supplier_list', function ($val, $data) use ($db) {
                return SupplierRelationUserModel::where([
                    'is_del' => $db::$is_del_normal,
                    'uid' => $data['uid']
                ])->field('supplierNo,supplierName,status')
                    ->select()
                    ->toArray();
            })
            ->select()
            ->toArray();

        return json_show(0, '获取成功', ['count' => $count, 'list' => $list]);

    }

    //修改供应商账号密码
    public static function changePassword(array $param = [])
    {

        $db = new SupplierUserModel();

        $res = $db
            ->where([
                'uid' => $param['uid'],
                'is_del' => $db::$is_del_normal,
            ])
            ->field('uid,password,salt,status,is_del')
            ->findOrEmpty()
            ->toArray();

        if (empty($res)) return json_show(1004, '该账号不存在');
        if ($res['status'] == $db::$status_disabled) return json_show(1004, '该账号已禁用');

        $old_password = get_encryption_password($param['old_password'], $res['salt']);
        if ($old_password['password'] != $res['password']) return json_show(1004, '密码错误');

        //更新密码
        $password = get_encryption_password($param['new_password']);

        $rs = $db
            ->where('uid', $res['uid'])
            ->save(['password' => $password['password'], 'salt' => $password['salt']]);

        return $rs ? json_show(0, '修改密码成功') : json_show(1005, '修改密码失败');

    }

    //获取供应商账号账号信息
    public static function readAccount(array $param = [])
    {
        $res = SupplierUser::field('uid,nickname,mobile,email')
            ->where([
                'uid' => $param['uid'],
                'is_del' => SupplierUser::$is_del_normal,
            ])
            ->append(['supplier_list'])
            ->withAttr('supplier_list', function ($val, $data) {

                return SupplierRelationUserModel::field('id,supplierNo,supplierName,status,addtime')
                    ->where([
                        'is_del' => SupplierUser::$is_del_normal,
                        'uid' => $data['uid']
                    ])
                    ->select()
                    ->toArray();
            })
            ->findOrEmpty()
            ->toArray();

        return json_show(0, '获取账号信息成功', $res);

    }

    //添加供应商账号
    public static function addAccount(array $param = [], string $token = '')
    {

        $user = GetUserInfo($token);

        //查找供应商名称
        $supplierName = Db::name('supplier')
            ->where(['code' => $param['supplierNo']])
            ->value('name', '');

        if ($supplierName == '') return json_show(1004, '该供应商名称不存在');

        Db::connect('mysql_sys')->startTrans();

        try {

            $db = new SupplierUserModel();

            $res = $db
                ->field('uid,status')
                ->where([
                    'is_del' => $db::$is_del_normal,
                    'mobile' => $param['mobile'],
                ])->findOrEmpty();

            $is_insert = true;
            if ($res->isEmpty()) {
                //新增账号
                $password = get_encryption_password(Config::get('app.default_password'));
                $res->uid = $db->insertGetId([
                    'nickname' => $param['nickname'],//姓名
                    'mobile' => $param['mobile'],//手机号
                    'email' => $param['email'],//邮箱
                    'password' => $password['password'],//密码密文
                    'salt' => $password['salt'],//盐值
                    'status' => $db::$status_normal,
                    'is_del' => $db::$is_del_normal,
                    'createrid' => $user['data']['user_id'] ?? 0,
                    'creater' => $user['data']['nickname'] ?? '',
                ]);

                $res->status = $db::$status_normal;

            } else {

                $rs = SupplierRelationUserModel::field('id')
                    ->where([
                        'is_del' => $db::$is_del_normal,
                        'uid' => $res->uid,
                        'supplierNo' => $param['supplierNo'],
                    ])->findOrEmpty()
                    ->isEmpty();

                if ($rs) throw new  Exception('不能重复添加');//该关联关系已存在，无需新增
                else $is_insert = false;

            }

            if ($is_insert) {
                SupplierRelationUserModel::create([
                    'uid' => $res->uid,
                    'supplierNo' => $param['supplierNo'],
                    'supplierName' => $supplierName,
                    'status' => $res->status,
                    'is_del' => $db::$is_del_normal,
                    'createrid' => $user['data']['user_id'] ?? 0,
                    'creater' => $user['data']['nickname'] ?? '',
                ])->save();
            }

            Db::connect('mysql_sys')->commit();
            return json_show(0, '操作成功');
        } catch (Exception $exception) {
            Db::connect('mysql_sys')->rollback();
            return json_show(1005, '操作失败' . $exception->getMessage());
        }

    }

    //修改供应商账号
    public static function editAccount(array $param = [])
    {

        $db = new SupplierUserModel();

        $res = $db
            ->field('uid')
            ->where(['uid' => $param['uid'], 'is_del' => $db::$is_del_normal])
            ->findOrEmpty()
            ->isEmpty();

        if ($res) return json_show(1004, '该账号不存在');

        $rs = $db
            ->where('uid', $param['uid'])
            ->save($param);

        return $rs ? json_show(0, '修改成功') : json_show(1005, '修改失败');

    }

    //修改供应商账号状态
    public static function statusAccount(array $param = [])
    {

        Db::connect('mysql_sys')->startTrans();

        try {

            $db = new SupplierUserModel();

            $res = $db
                ->field('uid,status')
                ->where([
                    'uid' => $param['uid'],
                    'is_del' => $db::$is_del_normal,
                ])->findOrEmpty();

            if ($res->isEmpty()) throw new Exception('该账号不存在');
            if ($res->status == $param['status']) throw new Exception('不能重复操作');

            $db->where([
                'uid' => $param['uid'],
                'is_del' => $db::$is_del_normal,
            ])->where('status', '<>', $param['status'])
                ->save(['status' => $param['status']]);

            SupplierRelationUserModel::where([
                'uid' => $param['uid'],
                'is_del' => $db::$is_del_normal,
            ])->where('status', '<>', $param['status'])
                ->save(['status' => $param['status']]);

            Db::connect('mysql_sys')->commit();
            return json_show(0, '操作成功');
        } catch (Exception $exception) {
            Db::connect('mysql_sys')->rollback();
            return json_show(1005, '操作失败,' . $exception->getMessage());
        }

    }

    //删除供应商账号
    public static function deleteAccount(int $uid = 0)
    {

        Db::connect('mysql_sys')->startTrans();

        try {

            $db = new SupplierUserModel();

            $res = $db
                ->field('uid')
                ->where([
                    'uid' => $uid,
                    'is_del' => $db::$is_del_normal,
                ])->findOrEmpty();

            if ($res->isEmpty()) throw new Exception('该账号不存在或已删除');

            $db->where('uid', $uid)
                ->where('is_del', $db::$is_del_normal)
                ->save(['is_del' => $db::$is_del_deleted]);

            SupplierRelationUserModel::where('uid', $uid)
                ->where('is_del', $db::$is_del_normal)
                ->save(['is_del' => $db::$is_del_deleted]);

            Db::connect('mysql_sys')->commit();
            return json_show(0, '删除成功');
        } catch (Exception $exception) {
            Db::connect('mysql_sys')->rollback();
            return json_show(1005, '删除失败,' . $exception->getMessage());
        }
    }

}